
/*
 * (c) Denis Smirnov <mithraen@freesource.info>, 2006
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/signal.h>
#include <math.h>

#define TEST_SECONDS 1
#define FLUSH_TIME 1000
#define TESTS 15

int open_pseudo()
{
	int fd;
	fd = open("/dev/dahdi/pseudo", O_RDWR);
	return fd;
}

int main()
{
    int fd;
    int res;
    int pass = 1;
    char buf[1024];
    double t_sec;
    double t_intervals;
    struct timeval start, now, allstart;
    long allcounts = 0;
    double jitter_max = 0, jitter_min = 1e37, jitter_sum = 0;

    fd = open_pseudo();
    if (fd < 0) {
        fprintf(stderr, "Unable to open DAHDI interface: %s\n", strerror(errno));
        return EXIT_FAILURE;
    }

    /* Flush input buffer */
    printf("Flushing input buffer...");
    fflush(stdout);
    res = read(fd, buf, FLUSH_TIME);
    printf("Ok\n");

    /* Show user message before get start time */
    printf("Test would be %d seconds long. Please wait\n", TEST_SECONDS * TESTS);

    gettimeofday(&now, NULL);
    allstart = start = now;
    for (int count = 0; pass <= TESTS;) {
        double errlvl;

        /* Reading for 1/8 of second */
        res = read(fd, buf, 1024);
        if (res < 0) {
            perror("Failed to read from pseudo interfaca");
            return EXIT_FAILURE;
        }
        count += res;
        if (count >= (TEST_SECONDS * 8000)) {
            int need_frames;
            double t_usec;

            gettimeofday(&now, NULL);
            t_usec = ((double) now.tv_sec - (double) start.tv_sec) * 1000000;
            t_usec += ((double) now.tv_usec - (double) start.tv_usec);
            t_sec = t_usec / 1000000;
            t_intervals = ceil(t_usec / 125);
            need_frames = (int) t_intervals;
            start = now;
            printf("%d samples in %f sec. (%d sample intervals) ", count, t_sec, need_frames);
            errlvl = 100.0 * ((double) count - t_intervals) / (double) count;
            double jitter = fabs(errlvl);

            jitter_sum += jitter;
            if (jitter > jitter_max)
                jitter_max = jitter;
            if (jitter < jitter_min)
                jitter_min = jitter;
            if (count == need_frames) {
                printf("Ok\n");
            } else if (abs(count - need_frames) == 1) {
                printf("Small error (1 frame) needs more time\n");
            } else {
                printf("%lf%% ", jitter);
                printf(errlvl > 0 ? "faster\n" : "longer\n");
            }
            pass++;
            allcounts += count;
            count = 0;
        }
    }
    long long t_usec = (long long) (now.tv_sec - allstart.tv_sec) * 1000000;

    t_usec += now.tv_usec - allstart.tv_usec;
    double need_frames = (double) t_usec / 125;
    const char *comment = (allcounts > need_frames) ? "unneeded" : "lost";

    printf("\n%ld frames in %lf interval\n", allcounts, need_frames);
    printf("8 %s frames (1 millisecond) every %lf seconds\n", comment, need_frames * 8 / fabs(allcounts - need_frames) / 8000);
    printf("Jitter:\nMin: %lf%%\nMax: %lf%%\nAvg: %lf%%\n", jitter_min, jitter_max, jitter_sum / TESTS);
}
